/*

    Gspoof v. 2.1 (September 0x02) - callbacks.c

    IMPORTANT
    From V. 2.1 Gspoof has been ported to libnet-1.1.x
    Old code (circa libnet-1.0.x) *will not work* with libnet-1.1.x and viceversa

    => To compile Gspoof >= v. 2.1 install libnet-1.1.x!
    => To compile Gspoof <= v. 2.0 install libnet-1.0.x!

 */

#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "common.h"

/* error buffer */
u_char ebuf[LIBNET_ERRBUF_SIZE];

/* boolean value */
u_short multipkg_boolean;
u_short datacontrol_boolean;
u_short linkl;

/* random value */
u_long seq;
u_long ack;
u_short id;
u_short urgp;

/* device (eth0, eth1, ppp0) */
char *device;

/* IP */
u_long shost, dhost;

/* index */
u_short i;

/* flags opt */
u_short furg, fack, fpsh, frst, fsyn, ffin;

/* libnet file context (new from libnet-1.1.x )*/
libnet_t *l;

/* probe for address and generate some random values */
void to_DEFAULT()
{
   struct libnet_ether_addr *ethaddr; /* eth address */

   /* reset some defaults value */
   furg = fack = fpsh = frst = fsyn = ffin = multipkg_boolean = linkl = datacontrol_boolean = 0;

   /* put default value */
   gtk_entry_set_text (GTK_ENTRY (ip_ttl_txt), "64");
   gtk_entry_set_text (GTK_ENTRY (ip_tos_txt), "8");
   gtk_entry_set_text (GTK_ENTRY (tcp_win_txt), "32767");
   gtk_entry_set_text (GTK_ENTRY (DELAY), "100");

   /* loading libnet core */
   if ((l=libnet_init(LIBNET_LINK, NULL, ebuf))==NULL)
     {
	sprintf(infoline, "Error creating libnet file context : %s\n", ebuf);
	info (infoline);
	return;
     }

   /* generate some random values */
   libnet_seed_prand(l);
   id = (u_short) libnet_get_prand(LIBNET_PRu16);
   seq = libnet_get_prand(LIBNET_PRu32);
   ack = libnet_get_prand(LIBNET_PRu32);
   urgp = (u_short) libnet_get_prand(LIBNET_PRu16);

   gtk_entry_set_text((GtkEntry *)ip_id_txt, ltostr(id));
   gtk_entry_set_text((GtkEntry *)tcp_seq_txt, ltostr(seq));
   gtk_entry_set_text((GtkEntry *)tcp_ack_txt, ltostr(ack));
   gtk_entry_set_text((GtkEntry *)tcp_urg_txt, ltostr(urgp));

   /* we probe for device */
   if ((device=libnet_getdevice(l))==NULL)
     {
	sprintf (infoline, "Error: cannot get device name : %s\n", libnet_geterror(l));
	info (infoline);
	return;
     }
   gtk_entry_set_text (GTK_ENTRY(eth_int_txt), device);

   /* we probe for device 's IP/MAC */
   shost = libnet_get_ipaddr4(l);
   if (shost == -1)
     {
	sprintf (infoline, "Error: autodetect device ip address failed: %s\n", libnet_geterror(l));
	info (infoline);
	return;
     }
   gtk_entry_set_text (GTK_ENTRY(ip_src_txt), libnet_addr2name4(shost, LIBNET_DONT_RESOLVE) );

   ethaddr = libnet_get_hwaddr(l);
   if (ethaddr == NULL)
     {
	sprintf (infoline, "Error: autodetect device MAC address failed: %s\n", libnet_geterror(l));
	info (infoline);
	return;
     }
   sprintf(infoline, "%X:%X:%X:%X:%X:%X", ethaddr->ether_addr_octet[0], ethaddr->ether_addr_octet[1], ethaddr->ether_addr_octet[2], ethaddr->ether_addr_octet[3], ethaddr->ether_addr_octet[4], ethaddr->ether_addr_octet[5]);
   gtk_entry_set_text (GTK_ENTRY(eth_src_txt), infoline);

   /* close network descriptor */
   libnet_destroy(l);
}

/* Procedure to send packet! */
void on_SENDBUT_clicked (GtkButton *button, gpointer user_data)
{
   u_short sport, dport;
   u_short flag;
   short w; /* byte written */
   u_short len; /* packet lenght */
   u_short datalen; /* data (tcp payload) lenght */
   u_char *data; /* data (tcp payload) */

   /* delay opt */
   u_short delay;
   u_short durate;
   u_long dcount;

   /* temp variable 4 eth addr */
   u_char *seth_temp;
   u_char *deth_temp;

   /* eth addr */
   u_char *seth;
   u_char *deth;

   /* eth type */
   u_char *ethtype;
   u_short evalue;

   /* other opt */
   u_short tos;
   u_short ttl;
   u_short win;

   u_short c;
   /* END of variable declaration */

   info ("\n");

   /* reading global and necessary variables */
   flag = 0x00;
   datalen = 0;
   data = NULL;

   tos = atoi (gtk_entry_get_text (GTK_ENTRY(ip_tos_txt)));
   ttl = atoi (gtk_entry_get_text (GTK_ENTRY(ip_ttl_txt)));
   win = atoi (gtk_entry_get_text (GTK_ENTRY(tcp_win_txt)));

   seq = atol (gtk_entry_get_text (GTK_ENTRY(tcp_seq_txt)));
   ack = atol (gtk_entry_get_text (GTK_ENTRY(tcp_ack_txt)));
   id = atoi (gtk_entry_get_text (GTK_ENTRY(ip_id_txt)));
   urgp = atoi (gtk_entry_get_text (GTK_ENTRY(tcp_urg_txt)));

   if (tos > 255 || atoi (gtk_entry_get_text (GTK_ENTRY(ip_tos_txt))) < 0)
     {
	info ("Error: TOS is not valid, must be between 0 and 255\n");
	return;
     }

   if (ttl > 255 || atoi (gtk_entry_get_text (GTK_ENTRY(ip_ttl_txt))) < 0)
     {
	info ("Error: TTL is not valid, must be between 0 and 255\n");
	return;
     }

   if (atol (gtk_entry_get_text (GTK_ENTRY(tcp_win_txt))) > 65535 || atoi (gtk_entry_get_text (GTK_ENTRY(tcp_win_txt))) < 0)
     {
	info ("Error: window value is not valid, it must be between 0 and 65535\n");
	return;
     }

   if (atol (gtk_entry_get_text (GTK_ENTRY(tcp_urg_txt))) > 65535 || atoi (gtk_entry_get_text (GTK_ENTRY(tcp_urg_txt))) < 0)
     {
	info ("Error: urgent pointer is not valid, it must be between 0 and 65535\n");
	return;
     }

   if (atol (gtk_entry_get_text (GTK_ENTRY(ip_id_txt))) > 65535 || atoi (gtk_entry_get_text (GTK_ENTRY(ip_id_txt))) < 0)
     {
	info ("Error: ID is not valid, it must be between 0 and 65535\n");
	return;
     }

   if (atoll (gtk_entry_get_text (GTK_ENTRY(tcp_seq_txt))) > 4294970000 || atoi (gtk_entry_get_text (GTK_ENTRY(tcp_seq_txt))) < 0)
     {
	info ("Error: sequence number is not valid, it must be between 0 and 2^32-1\n");
	return;
     }

   if (atoll (gtk_entry_get_text (GTK_ENTRY(tcp_ack_txt))) > 4294970000 || atoi (gtk_entry_get_text (GTK_ENTRY(tcp_ack_txt))) < 0)
     {
	info ("Error: acknowledgement number is not valid, it must be between 0 and 2^32-1\n");
	return;
     }

   /* delay options */
   delay = atoi(gtk_entry_get_text(GTK_ENTRY(DELAY)));
   durate = atoi(gtk_entry_get_text(GTK_ENTRY(DURATE)));

   /* eth type */
   ethtype = gtk_entry_get_text (GTK_ENTRY (combo_entry1));
   evalue = 0x0800;  /* IP value = default */
   if (strcmp(ethtype, "IP")) evalue = 0x9000; /*strcmp return 0 if strings are equal! */

   /* read ports numbers */
   sport = atoi(gtk_entry_get_text(GTK_ENTRY(tcp_src_txt)));
   if (sport < 1 || atol(gtk_entry_get_text(GTK_ENTRY(tcp_src_txt)))>65535)
     {
	info ("Error: source port is not valid\n");
	return;
     }
   dport = atoi(gtk_entry_get_text (GTK_ENTRY(tcp_dst_txt)));
   if (dport < 1 || atol(gtk_entry_get_text(GTK_ENTRY(tcp_dst_txt)))>65535)
     {
	info ("Error: destination port is not valid\n");
	return;
     }

   /* control flags */
   if (ffin) flag+=0x01;
   if (fsyn) flag+=0x02;
   if (frst) flag+=0x04;
   if (fpsh) flag+=0x08;
   if (fack) flag+=0x10;
   if (furg) flag+=0x20;

   /* read payload if present */
   if (datacontrol_boolean)
     {
	data = gtk_entry_get_text(GTK_ENTRY(DATA));
	datalen = strlen (data);
     }

   /* reading device */
   device = gtk_entry_get_text(GTK_ENTRY(eth_int_txt));
   if (device == NULL)
     {
	info ("Error: device is not valid\n");
	return;
     }

     /* load libnet core, new! */
   if (linkl)
     l = libnet_init (LIBNET_LINK, device, ebuf);
   else
     l = libnet_init (LIBNET_RAW4, NULL, ebuf);

   if (l==NULL)
     {
	sprintf (infoline, "Error creating libnet file context : %s\n", ebuf);
	info (infoline);
	return;
     }

   /* reading IP forms */
   if ((shost = libnet_name2addr4(l, gtk_entry_get_text (GTK_ENTRY(ip_src_txt)), LIBNET_RESOLVE))==-1)
     {
	info ("Error: source ip address is not valid\n");
	return;
     }
   if ((dhost = libnet_name2addr4(l, gtk_entry_get_text (GTK_ENTRY(ip_dst_txt)), LIBNET_RESOLVE))==-1)
     {
	info ("Error: destination ip address is not valid\n");
	return;
     }

   /* total packet lenght */
   len = LIBNET_ETH_H + LIBNET_IPV4_H + LIBNET_TCP_H + datalen;
   /* linkl=0 -> len = len - LIBNET_ETH_H (LIBNET_ETH_H is build by kernel and not libnet! */

   switch (linkl)
     {
      case 0:
	/* printf libnet ok */
	if (l!=NULL)
	  info ("* Libnet file context created\n");

	/* TCP */
	if (libnet_build_tcp(sport, dport, seq, ack, flag, win, 0, urgp, LIBNET_TCP_H + datalen, data, datalen, l, 0)==-1)
	  {
	     sprintf (infoline,"Error during TCP header creation : %s\n", libnet_geterror(l));
	     info(infoline);
	     break;
	  }
	info ("* TCP header build\n");

	/* IP */
	if (libnet_build_ipv4(LIBNET_TCP_H + LIBNET_IPV4_H + datalen, tos, id, 0, ttl, IPPROTO_TCP, 0, shost, dhost, NULL, 0, l, 0)==-1)
	  {
	     sprintf (infoline, "Error during IP header creation : %s\n", libnet_geterror(l));
	     info(infoline);
	     break;
	  }
	info("* IP header build\n");

        if (multipkg_boolean)
	  {
	     sprintf (infoline, "** Writing %d ms spaced packets for %d seconds: ", delay, durate);
	     info (infoline);

	     dcount = c = 0;
	     while (dcount < durate*1000)
	       {
		  c++;
		  w = libnet_write(l);
		  if (w==-1)
		    {
		       sprintf (infoline, "Error: %s\n", libnet_geterror(l));
		       info (infoline);
		       break;
		    }
		  info (". ");
		  usleep(delay*1000); /*ms! */
		  dcount+=delay;
	       }
	     sprintf (infoline, "DonE! (%d * %d byte)\n", c, len);
	     info (infoline);
	  }
	else
	  {
	     w = libnet_write(l);
	     if (w==-1)
	       {
		  sprintf (infoline, "Error: %s\n", libnet_geterror(l));
		  info (infoline);
		  break;
	       }
	     sprintf(infoline, "** Packet has been correctly send (total %d bytes)\n", len);
	     info (infoline);
	  }

       /* at the end we close raw_socket */
	libnet_destroy(l);
	break;

      case 1:
	/* read MAC from GTK */
	seth_temp = gtk_entry_get_text(GTK_ENTRY(eth_src_txt));
	deth_temp = gtk_entry_get_text(GTK_ENTRY(eth_dst_txt));

	seth = malloc (6*sizeof(u_char));
	deth = malloc (6*sizeof(u_char));

	seth = emb_hex_aton (seth_temp);
	if (seth==NULL)
	  {
	     info ("Source MAC address is not valid\n");
	     break;
	  }
	deth = emb_hex_aton (deth_temp);
	if (deth==NULL)
	  {
	     info ("Destination MAC address is not valid\n");
	     break;
	  }

	/* print libnet and device ok */
	if (l!=NULL)
	  info ("* Libnet file context created\n");
	if (device!=NULL)
	  {
	     sprintf(infoline, "* Interface %s correctly opened\n", device);
	     info (infoline);
	  }

        /* TCP creation */
	if (libnet_build_tcp(sport, dport, seq, ack, flag, win, 0, urgp, LIBNET_TCP_H + datalen, data, datalen, l, 0)==-1)
	  {
	     sprintf (infoline, "Error during TCP header creation : %s\n", libnet_geterror(l));
	     info (infoline);
	     break;
	  }
	info ("* TCP header build\n");

	/* IP creation */
	if (libnet_build_ipv4(LIBNET_IPV4_H + LIBNET_TCP_H + datalen, tos, id, 0, ttl, IPPROTO_TCP, 0, shost, dhost, NULL, 0, l, 0)==-1)
	  {
	     sprintf (infoline, "Error during IP header creation : %s\n", libnet_geterror(l));
	     info (infoline);
	     break;
	  }
	info ("* IP header build\n");

	/* ETH creation */
	if (libnet_build_ethernet(deth, seth, evalue, NULL, 0, l, 0)==-1)
	  {
	     sprintf (infoline, "Error during ethernet header creation : %s\n", libnet_geterror(l));
	     info (infoline);
	     break;
	  }
	info ("* Ethernet frame build\n");

	info ("\n");

        if (multipkg_boolean)
	  {
	     sprintf (infoline, "** Writing %d ms spaced packets for %d seconds: ", delay, durate);
	     info (infoline);

	     dcount = c = 0;
	     while (dcount < durate*1000)
	       {
		  c++;
		  w = libnet_write(l);
		  if (w==-1)
		    {
		       sprintf (infoline, "Error: %s\n",libnet_geterror(l));
		       info (infoline);
		       break;
		    }
		  info (". ");
		  usleep(delay*1000); /*ms! */
		  dcount+=delay;
	       }
	     sprintf (infoline, "DonE! (%d * %d byte)\n", c, len);
	     info (infoline);
	  }
	else
	  {
	     w = libnet_write(l);
	     if (w==-1)
	       {
		  sprintf (infoline, "Error: %s\n",libnet_geterror(l));
		  info (infoline);
		  break;
	       }
	     sprintf(infoline, "\n** Packet has been correctly send (total %d bytes)\n", len);
	     info (infoline);
	  }

       /* at the end we close raw_socket */
	libnet_destroy(l);
	break;
     }
   /* end of switch */
   return;
}

/* others events */
void on_MULTIPKG_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
   if (!multipkg_boolean)
     {
	multipkg_boolean=1;
	gtk_entry_set_editable (GTK_ENTRY (DELAY), TRUE);
	gtk_entry_set_editable (GTK_ENTRY (DURATE), TRUE);
     }
   else
     {
	multipkg_boolean=0;
	gtk_entry_set_editable (GTK_ENTRY (DELAY), FALSE);
	gtk_entry_set_editable (GTK_ENTRY (DURATE), FALSE);
     }
}

void on_DATACONTROL_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
   if (!datacontrol_boolean)
     {
	datacontrol_boolean=1;
	gtk_entry_set_editable (GTK_ENTRY (DATA), TRUE);
     }
   else
     {
	datacontrol_boolean=0;
	gtk_entry_set_editable (GTK_ENTRY (DATA), FALSE);
     }
}

void on_togglebutton1_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
   if (!linkl)
     {
	linkl=1;
	gtk_entry_set_editable (GTK_ENTRY (eth_int_txt), TRUE);
	gtk_entry_set_editable (GTK_ENTRY (eth_src_txt), TRUE);
	gtk_entry_set_editable (GTK_ENTRY (eth_dst_txt), TRUE);
     }
   else
     {
	linkl=0;
	gtk_entry_set_editable (GTK_ENTRY (eth_int_txt), FALSE);
	gtk_entry_set_editable (GTK_ENTRY (eth_src_txt), FALSE);
	gtk_entry_set_editable (GTK_ENTRY (eth_dst_txt), FALSE);
     }
}

/* flags */
void on_URGCHECK_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
   if (!furg) furg=1;
   else furg=0;
}

void on_ACKCHECK_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
   if (!fack) fack=1;
   else fack=0;
}

void on_PSHCHECK_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
   if (!fpsh) fpsh=1;
   else fpsh=0;
}

void on_RSTCHECK_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
   if (!frst) frst=1;
   else frst=0;
}

void on_SYNCHECK_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
   if (!fsyn) fsyn=1;
   else fsyn=0;
}

void on_FINCHECK_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
   if (!ffin) ffin=1;
   else ffin=0;
}

/* resetting values */
void on_RESETBUT_clicked (GtkButton *button, gpointer user_data)
{
   to_DEFAULT();
   info ("\nRestoring default value done\n");
}

/* close application */
void on_QUIT_BUTTON_clicked (GtkButton *button, gpointer user_data)
{
   gtk_main_quit();
}
void delete_event(GtkWidget *widget, gpointer data)
{
   gtk_main_quit();
}

/* print info line */
void info (char * line)
{
   gtk_text_insert((GtkText *) INFO_TXT, NULL, NULL, NULL, line, -1);
}

/* print credits */
void on_CREDIT_BUTTON_clicked (GtkButton *button, gpointer user_data)
{
   info ("\nGspoof 2.1 - Written in C and GTK under Slackware Linux, September 0x02, Embyte (embyte@madlab.it)\n");
}
